@@ -12,6 +12,7 @@ if sys.version_info[0] >= 3:   # pragma: no cover
     import configparser
     from urllib.parse import urlparse
     from collections import UserDict
+    from inspect import getfullargspec as getargspec
 else:  # pragma: no cover
     string_types = basestring,
     text_type = unicode
@@ -20,3 +21,4 @@ else:  # pragma: no cover
     import ConfigParser as configparser
     from urlparse import urlparse
     from UserDict import UserDict
+    from inspect import getargspec
