@@ -1,20 +1,24 @@
+"""$ rio rasterize"""
+
+
 import json
 import logging
 from math import ceil
 import os
 
+from affine import Affine
 import click
 import cligj
 
-from .helpers import resolve_inout
-from . import options
 import rasterio
 from rasterio.errors import CRSError
-from rasterio.transform import Affine
 from rasterio.coords import disjoint_bounds
+from rasterio.rio import options
+from rasterio.rio.helpers import resolve_inout
+import rasterio.shutil
 
 
-logger = logging.getLogger('rio')
+logger = logging.getLogger(__name__)
 
 
 # Common options used below
@@ -24,7 +28,8 @@ files_inout_arg = click.argument(
     'files',
     nargs=-1,
     type=click.Path(resolve_path=True),
-    metavar="INPUTS... OUTPUT")
+    metavar="INPUTS... OUTPUT",
+    callback=options.files_inout_handler)
 
 
 @click.command(short_help='Rasterize features.')
@@ -49,7 +54,8 @@ files_inout_arg = click.argument(
 @click.option('--property', 'prop', type=str, default=None, help='Property in '
               'GeoJSON features to use for rasterized values.  Any features '
               'that lack this property will be given --default_value instead.')
-@options.force_overwrite_opt
+@options.overwrite_opt
+@options.nodata_opt
 @options.creation_options
 @click.pass_context
 def rasterize(
@@ -66,55 +72,56 @@ def rasterize(
         default_value,
         fill,
         prop,
-        force_overwrite,
+        overwrite,
+        nodata,
         creation_options):
     """Rasterize GeoJSON into a new or existing raster.
 
-    If the output raster exists, rio-rasterize will rasterize feature values
-    into all bands of that raster.  The GeoJSON is assumed to be in the same
-    coordinate reference system as the output unless --src-crs is provided.
-
-    --default_value or property values when using --property must be using a
-    data type valid for the data type of that raster.
-
-
-    If a template raster is provided using the --like option, the affine
-    transform and data type from that raster will be used to create the output.
-    Only a single band will be output.
-
-    The GeoJSON is assumed to be in the same coordinate reference system unless
+    If the output raster exists, rio-rasterize will rasterize feature
+    values into all bands of that raster.  The GeoJSON is assumed to be
+    in the same coordinate reference system as the output unless
     --src-crs is provided.
 
-    --default_value or property values when using --property must be using a
-    data type valid for the data type of that raster.
+    --default_value or property values when using --property must be
+    using a data type valid for the data type of that raster.
 
-    --driver, --bounds, --dimensions, and --res are ignored when output exists
-    or --like raster is provided
+    If a template raster is provided using the --like option, the affine
+    transform and data type from that raster will be used to create the
+    output.  Only a single band will be output.
 
+    The GeoJSON is assumed to be in the same coordinate reference system
+    unless --src-crs is provided.
 
-    If the output does not exist and --like raster is not provided, the input
-    GeoJSON will be used to determine the bounds of the output unless
-    provided using --bounds.
+    --default_value or property values when using --property must be
+    using a data type valid for the data type of that raster.
+
+    --driver, --bounds, --dimensions, --res, --nodata are ignored when
+    output exists or --like raster is provided
+
+    If the output does not exist and --like raster is not provided, the
+    input GeoJSON will be used to determine the bounds of the output
+    unless provided using --bounds.
 
     --dimensions or --res are required in this case.
 
     If --res is provided, the bottom and right coordinates of bounds are
     ignored.
 
+    Note
+    ----
+
+    The GeoJSON is not projected to match the coordinate reference
+    system of the output or --like rasters at this time.  This
+    functionality may be added in the future.
 
-    Note:
-    The GeoJSON is not projected to match the coordinate reference system
-    of the output or --like rasters at this time.  This functionality may be
-    added in the future.
     """
+
     from rasterio.crs import CRS
     from rasterio.features import rasterize
     from rasterio.features import bounds as calculate_bounds
 
-    verbosity = (ctx.obj and ctx.obj.get('verbosity')) or 1
-
     output, files = resolve_inout(
-        files=files, output=output, force_overwrite=force_overwrite)
+        files=files, output=output, overwrite=overwrite)
 
     bad_param = click.BadParameter('invalid CRS.  Must be an EPSG code.',
                                    ctx, param=src_crs, param_hint='--src_crs')
@@ -131,7 +138,7 @@ def rasterize(
     if fill == int(fill):
         fill = int(fill)
 
-    with rasterio.Env(CPL_DEBUG=verbosity > 2):
+    with ctx.obj['env']:
 
         def feature_value(feature):
             if prop and 'properties' in feature:
@@ -152,7 +159,7 @@ def rasterize(
 
         geojson_bounds = geojson.get('bbox', calculate_bounds(geojson))
 
-        if os.path.exists(output):
+        if rasterio.shutil.exists(output):
             with rasterio.open(output, 'r+') as out:
                 if has_src_crs and src_crs != out.crs:
                     raise click.BadParameter('GeoJSON does not match crs of '
@@ -165,7 +172,7 @@ def rasterize(
                                "reference systems?",
                                err=True)
 
-                meta = out.meta.copy()
+                meta = out.meta
 
                 result = rasterize(
                     geometries,
@@ -181,7 +188,8 @@ def rasterize(
                     # Burn in any non-fill pixels, and update mask accordingly
                     ne = result != fill
                     data[ne] = result[ne]
-                    data.mask[ne] = False
+                    if data.mask.any():
+                        data.mask[ne] = False
                     out.write(data, indexes=bidx)
 
         else:
@@ -199,13 +207,9 @@ def rasterize(
                                "systems?",
                                err=True)
 
-                kwargs = template_ds.meta.copy()
+                kwargs = template_ds.profile
                 kwargs['count'] = 1
-
-                # DEPRECATED
-                # upgrade transform to affine object or we may get an invalid
-                # transform set on output
-                kwargs['transform'] = template_ds.affine
+                kwargs['transform'] = template_ds.transform
 
                 template_ds.close()
 
@@ -250,12 +254,16 @@ def rasterize(
                                         bounds[3]),
                     'driver': driver
                 }
-                kwargs.update(**creation_options)
+
+            kwargs.update(**creation_options)
+
+            if nodata is not None:
+                kwargs['nodata'] = nodata
 
             result = rasterize(
                 geometries,
                 out_shape=(kwargs['height'], kwargs['width']),
-                transform=kwargs.get('affine', kwargs['transform']),
+                transform=kwargs['transform'],
                 all_touched=all_touched,
                 dtype=kwargs.get('dtype', None),
                 default_value=default_value,
@@ -264,7 +272,5 @@ def rasterize(
             if 'dtype' not in kwargs:
                 kwargs['dtype'] = result.dtype
 
-            kwargs['nodata'] = fill
-
             with rasterio.open(output, 'w', **kwargs) as out:
                 out.write(result, indexes=1)
