@@ -1,18 +1,22 @@
+"""$ rio warp"""
+
 import logging
-from math import ceil
+from math import ceil, log
 import warnings
 
 import click
-from cligj import files_inout_arg, format_opt
+from cligj import format_opt
 
-from .helpers import resolve_inout
-from . import options
 import rasterio
 from rasterio.crs import CRS
+from rasterio.env import setenv, GDALVersion
 from rasterio.errors import CRSError
+from rasterio.rio import options
+from rasterio.rio.helpers import resolve_inout
 from rasterio.transform import Affine
 from rasterio.warp import (
-    reproject, Resampling, calculate_default_transform, transform_bounds)
+    reproject, Resampling, SUPPORTED_RESAMPLING, transform_bounds,
+    aligned_target, calculate_default_transform as calcdt)
 
 
 # Improper usage of rio-warp can lead to accidental creation of
@@ -22,29 +26,8 @@ MAX_OUTPUT_WIDTH = 100000
 MAX_OUTPUT_HEIGHT = 100000
 
 
-def bounds_handler(ctx, param, value):
-    """Warn about future usage changes."""
-    if value:
-        click.echo(
-            "Future Warning: "
-            "the semantics of the `--bounds` option will change in Rasterio "
-            "version 1.0 from bounds of the source dataset to bounds of the "
-            "destination dataset.", err=True)
-    return value
-
-
-def x_dst_bounds_handler(ctx, param, value):
-    """Warn about future usage changes."""
-    if value:
-        click.echo(
-            "Future Warning: "
-            "the `--x-dst-bounds` option will be removed in Rasterio version "
-            "1.0 in favor of `--bounds`.", err=True)
-    return value
-
-
 @click.command(short_help='Warp a raster dataset.')
-@files_inout_arg
+@options.files_inout_arg
 @options.output_opt
 @format_opt
 @click.option(
@@ -59,20 +42,13 @@ def x_dst_bounds_handler(ctx, param, value):
     '--src-bounds',
     nargs=4, type=float, default=None,
     help="Determine output extent from source bounds: left bottom right top "
-         "(note: for future backwards compatibility in 1.0).")
+         ". Cannot be used with destination --bounds")
 @click.option(
-    '--x-dst-bounds',
-    nargs=4, type=float, default=None, callback=x_dst_bounds_handler,
-    help="Set output extent from bounding values: left bottom right top "
-         "(note: this option will be removed in 1.0).")
-@click.option(
-    '--bounds',
-    nargs=4, type=float, default=None, callback=bounds_handler,
-    help="Determine output extent from source bounds: left bottom right top "
-         "(note: the semantics of this option will change to those of "
-         "`--x-dst-bounds` in version 1.0).")
+    '--bounds', '--dst-bounds', nargs=4, type=float, default=None,
+    help="Determine output extent from destination bounds: left bottom right top")
 @options.resolution_opt
-@click.option('--resampling', type=click.Choice([r.name for r in Resampling]),
+@click.option('--resampling',
+              type=click.Choice([r.name for r in SUPPORTED_RESAMPLING]),
               default='nearest', help="Resampling method.",
               show_default=True)
 @click.option('--src-nodata', default=None, show_default=True,
@@ -81,21 +57,25 @@ def x_dst_bounds_handler(ctx, param, value):
               type=float, help="Manually override destination nodata")
 @click.option('--threads', type=int, default=1,
               help='Number of processing threads.')
-@click.option('--check-invert-proj', type=bool, default=True,
+@click.option('--check-invert-proj/--no-check-invert-proj', default=True,
               help='Constrain output to valid coordinate region in dst-crs')
-@options.force_overwrite_opt
+@click.option('--target-aligned-pixels/--no-target-aligned-pixels', default=False,
+              help='align the output bounds based on the resolution')
+@options.overwrite_opt
 @options.creation_options
 @click.pass_context
 def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
-         x_dst_bounds, bounds, res, resampling, src_nodata, dst_nodata, threads, check_invert_proj,
-         force_overwrite, creation_options):
+         dst_bounds, res, resampling, src_nodata, dst_nodata, threads,
+         check_invert_proj, overwrite, creation_options,
+         target_aligned_pixels):
     """
     Warp a raster dataset.
 
     If a template raster is provided using the --like option, the
     coordinate reference system, affine transform, and dimensions of
     that raster will be used for the output.  In this case --dst-crs,
-    --bounds, --res, and --dimensions options are ignored.
+    --bounds, --res, and --dimensions options are not applicable and
+    an exception will be raised.
 
     \b
         $ rio warp input.tif output.tif --like template.tif
@@ -112,7 +92,8 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
     \b
         --dst-crs '{"proj": "utm", "zone": 18, ...}'
 
-    If --dimensions are provided, --res and --bounds are ignored.
+    If --dimensions are provided, --res and --bounds are not applicable and an
+    exception will be raised.
     Resolution is calculated based on the relationship between the
     raster bounds in the target coordinate system and the dimensions,
     and may produce rectangular rather than square pixels.
@@ -129,10 +110,8 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
         > --bounds -78 22 -76 24 --res 0.1 --dst-crs EPSG:4326
 
     """
-    verbosity = (ctx.obj and ctx.obj.get('verbosity')) or 1
-
     output, files = resolve_inout(
-        files=files, output=output, force_overwrite=force_overwrite)
+        files=files, output=output, overwrite=overwrite)
 
     resampling = Resampling[resampling]  # get integer code for method
 
@@ -143,25 +122,47 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
         # Expand one value to two if needed
         res = (res[0], res[0]) if len(res) == 1 else res
 
-    with rasterio.Env(CPL_DEBUG=verbosity > 2,
-                      CHECK_WITH_INVERT_PROJ=check_invert_proj):
+    if target_aligned_pixels:
+        if not res:
+            raise click.BadParameter(
+                '--target-aligned-pixels requires a specified resolution')
+        if src_bounds or dst_bounds:
+            raise click.BadParameter(
+                '--target-aligned-pixels cannot be used with '
+                '--src-bounds or --dst-bounds')
+
+    # Check invalid parameter combinations
+    if like:
+        invalid_combos = (dimensions, dst_bounds, dst_crs, res)
+        if any(p for p in invalid_combos if p is not None):
+            raise click.BadParameter(
+                "--like cannot be used with any of --dimensions, --bounds, "
+                "--dst-crs, or --res")
+
+    elif dimensions:
+        invalid_combos = (dst_bounds, res)
+        if any(p for p in invalid_combos if p is not None):
+            raise click.BadParameter(
+                "--dimensions cannot be used with --bounds or --res")
+
+    with ctx.obj['env']:
+        setenv(CHECK_WITH_INVERT_PROJ=check_invert_proj)
+
         with rasterio.open(files[0]) as src:
             l, b, r, t = src.bounds
-            out_kwargs = src.meta.copy()
+            out_kwargs = src.profile.copy()
             out_kwargs['driver'] = driver
 
             # Sort out the bounds options.
-            src_bounds = bounds or src_bounds
-            dst_bounds = x_dst_bounds
             if src_bounds and dst_bounds:
                 raise click.BadParameter(
-                    "Source and destination bounds may not be specified "
-                    "simultaneously.")
+                    "--src-bounds and destination --bounds may not be "
+                    "specified simultaneously.")
 
             if like:
                 with rasterio.open(like) as template_ds:
                     dst_crs = template_ds.crs
-                    dst_transform = template_ds.affine
+                    dst_transform = template_ds.transform
                     dst_height = template_ds.height
                     dst_width = template_ds.width
 
@@ -176,9 +177,10 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
                     # Calculate resolution appropriate for dimensions
                     # in target.
                     dst_width, dst_height = dimensions
+                    bounds = src_bounds or src.bounds
                     try:
                         xmin, ymin, xmax, ymax = transform_bounds(
-                            src.crs, dst_crs, *src.bounds)
+                            src.crs, dst_crs, *bounds)
                     except CRSError as err:
                         raise click.BadParameter(
                             str(err), param='dst_crs', param_hint='dst_crs')
@@ -212,16 +214,24 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
 
                 else:
                     try:
-                        dst_transform, dst_width, dst_height = calculate_default_transform(
-                            src.crs, dst_crs, src.width, src.height,
-                            *src.bounds, resolution=res)
+                        if src.transform.is_identity and src.gcps:
+                            src_crs = src.gcps[1]
+                            kwargs = {'gcps': src.gcps[0]}
+                        else:
+                            src_crs = src.crs
+                            kwargs = src.bounds._asdict()
+                        dst_transform, dst_width, dst_height = calcdt(
+                            src_crs, dst_crs, src.width, src.height,
+                            resolution=res, **kwargs)
                     except CRSError as err:
                         raise click.BadParameter(
                             str(err), param='dst_crs', param_hint='dst_crs')
+
             elif dimensions:
                 # Same projection, different dimensions, calculate resolution.
                 dst_crs = src.crs
                 dst_width, dst_height = dimensions
+                l, b, r, t = src_bounds or (l, b, r, t)
                 dst_transform = Affine(
                     (r - l) / float(dst_width),
                     0, l, 0,
@@ -233,7 +243,7 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
                 # Same projection, different dimensions and possibly
                 # different resolution.
                 if not res:
-                    res = (src.affine.a, -src.affine.e)
+                    res = (src.transform.a, -src.transform.e)
 
                 dst_crs = src.crs
                 xmin, ymin, xmax, ymax = (src_bounds or dst_bounds)
@@ -250,10 +260,13 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
 
             else:
                 dst_crs = src.crs
-                dst_transform = src.affine
+                dst_transform = src.transform
                 dst_width = src.width
                 dst_height = src.height
 
+            if target_aligned_pixels:
+                dst_transform, dst_width, dst_height = aligned_target(dst_transform, dst_width, dst_height, res)
+
             # If src_nodata is not None, update the dst metadata NODATA
             # value to src_nodata (will be overridden by dst_nodata if it is not None
             if src_nodata is not None:
@@ -270,9 +283,7 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
                         "--src-nodata must be provided because dst-nodata is not None")
                 else:
                     # Update the dst nodata value
-                    out_kwargs.update({
-                        'nodata': dst_nodata
-                        })
+                    out_kwargs.update({'nodata': dst_nodata})
 
             # When the bounds option is misused, extreme values of
             # destination width and height may result.
@@ -286,24 +297,30 @@ def warp(ctx, files, output, driver, like, dst_crs, dimensions, src_bounds,
             out_kwargs.update({
                 'crs': dst_crs,
                 'transform': dst_transform,
-                'affine': dst_transform,
                 'width': dst_width,
                 'height': dst_height
             })
 
+            # Adjust block size if necessary.
+            if ('blockxsize' in out_kwargs and
+                    dst_width < out_kwargs['blockxsize']):
+                del out_kwargs['blockxsize']
+            if ('blockysize' in out_kwargs and
+                    dst_height < out_kwargs['blockysize']):
+                del out_kwargs['blockysize']
+
             out_kwargs.update(**creation_options)
 
             with rasterio.open(output, 'w', **out_kwargs) as dst:
-                for i in range(1, src.count + 1):
-
-                    reproject(
-                        source=rasterio.band(src, i),
-                        destination=rasterio.band(dst, i),
-                        src_transform=src.affine,
-                        src_crs=src.crs,
-                        src_nodata=src_nodata,
-                        dst_transform=out_kwargs['transform'],
-                        dst_crs=out_kwargs['crs'],
-                        dst_nodata=dst_nodata,
-                        resampling=resampling,
-                        num_threads=threads)
+                reproject(
+                    source=rasterio.band(src, list(range(1, src.count + 1))),
+                    destination=rasterio.band(
+                        dst, list(range(1, src.count + 1))),
+                    src_transform=src.transform,
+                    src_crs=src.crs,
+                    src_nodata=src_nodata,
+                    dst_transform=out_kwargs['transform'],
+                    dst_crs=out_kwargs['crs'],
+                    dst_nodata=dst_nodata,
+                    resampling=resampling,
+                    num_threads=threads)
