@@ -1,267 +1,18 @@
 import json
-import logging
-import sys
 
-import click
-from click import Context
 from click.testing import CliRunner
 import pytest
 
+import numpy as np
 import rasterio
-from rasterio.rio import info
-from rasterio.rio.edit_info import (edit, all_handler, crs_handler,
-                                    tags_handler, transform_handler)
 from rasterio.rio.main import main_group
+from rasterio.env import GDALVersion
 
+from .conftest import requires_gdal21
 
-logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
 
-
-def test_edit_nodata_err(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--nodata', '-1'])
-    assert result.exit_code == 2
-
-
-def test_edit_nodata(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--nodata', '255'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.nodata == 255.0
-
-
-def test_edit_crs_err(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--crs', 'LOL:WUT'])
-    assert result.exit_code == 2
-
-
-def test_edit_crs_epsg(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--crs', 'EPSG:32618'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32618'}
-
-
-def test_edit_crs_proj4(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--crs', '+init=epsg:32618'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32618'}
-
-
-def test_edit_crs_obj(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(
-        edit, [inputfile, '--crs', '{"init": "epsg:32618"}'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs.to_dict() == {'init': 'epsg:32618'}
-
-
-def test_edit_transform_err_not_json(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--transform', 'LOL'])
-    assert result.exit_code == 2
-
-
-def test_edit_transform_err_bad_array(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [inputfile, '--transform', '[1,2]'])
-    assert result.exit_code == 2
-
-
-def test_edit_transform_affine(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    input_t = '[300.038, 0.0, 101985.0, 0.0, -300.042, 2826915.0]'
-    result = runner.invoke(edit, [inputfile, '--transform', input_t])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        for a, b in zip(src.affine, json.loads(input_t)):
-            assert round(a, 6) == round(b, 6)
-
-
-def test_edit_transform_gdal(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    input_t = '[300.038, 0.0, 101985.0, 0.0, -300.042, 2826915.0]'
-    result = runner.invoke(edit, [
-        inputfile,
-        '--transform', '[101985.0, 300.038, 0.0, 2826915.0, 0.0, -300.042]'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        for a, b in zip(src.affine, json.loads(input_t)):
-            assert round(a, 6) == round(b, 6)
-
-
-def test_edit_tags(data):
-    runner = CliRunner()
-    inputfile = str(data.join('RGB.byte.tif'))
-    result = runner.invoke(edit, [
-        inputfile, '--tag', 'lol=1', '--tag', 'wut=2'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.tags()['lol'] == '1'
-        assert src.tags()['wut'] == '2'
-
-
-class MockContext:
-
-    def __init__(self):
-        self.obj = {}
-
-
-class MockOption:
-
-    def __init__(self, name):
-        self.name = name
-
-
-def test_all_callback_pass(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'transform': 'foo'}
-    assert all_handler(ctx, None, None) is None
-
-
-def test_all_callback(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'transform': 'foo'}
-    assert all_handler(ctx, None, True) == {'transform': 'foo'}
-
-
-def test_all_callback_None(data):
-    ctx = MockContext()
-    assert all_handler(ctx, None, None) is None
-
-
-def test_transform_callback_pass(data):
-    """Always return None if the value is None"""
-    ctx = MockContext()
-    ctx.obj['like'] = {'transform': 'foo'}
-    assert transform_handler(ctx, MockOption('transform'), None) is None
-
-
-def test_transform_callback_err(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'transform': 'foo'}
-    with pytest.raises(click.BadParameter):
-        transform_handler(ctx, MockOption('transform'), '?')
-
-
-def test_transform_callback(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'transform': 'foo'}
-    assert transform_handler(ctx, MockOption('transform'), 'like') == 'foo'
-
-
-def test_crs_callback_pass(data):
-    """Always return None if the value is None."""
-    ctx = MockContext()
-    ctx.obj['like'] = {'crs': 'foo'}
-    assert crs_handler(ctx, MockOption('crs'), None) is None
-
-
-def test_crs_callback(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'crs': 'foo'}
-    assert crs_handler(ctx, MockOption('crs'), 'like') == 'foo'
-
-
-def test_tags_callback_err(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'tags': {'foo': 'bar'}}
-    with pytest.raises(click.BadParameter):
-        tags_handler(ctx, MockOption('tags'), '?') == {'foo': 'bar'}
-
-
-def test_tags_callback(data):
-    ctx = MockContext()
-    ctx.obj['like'] = {'tags': {'foo': 'bar'}}
-    assert tags_handler(ctx, MockOption('tags'), 'like') == {'foo': 'bar'}
-
-
-def test_edit_crs_like(data):
-    runner = CliRunner()
-
-    # Set up the file to be edited.
-    inputfile = str(data.join('RGB.byte.tif'))
-    with rasterio.open(inputfile, 'r+') as dst:
-        dst.crs = {'init': 'epsg:32617'}
-        dst.nodata = 1.0
-
-    # Double check.
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32617'}
-        assert src.nodata == 1.0
-
-    # The test.
-    templatefile = 'tests/data/RGB.byte.tif'
-    result = runner.invoke(
-        main_group,
-        ['edit-info', inputfile, '--like', templatefile, '--crs', 'like'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32618'}
-        assert src.nodata == 1.0
-
-
-def test_edit_nodata_like(data):
-    runner = CliRunner()
-
-    # Set up the file to be edited.
-    inputfile = str(data.join('RGB.byte.tif'))
-    with rasterio.open(inputfile, 'r+') as dst:
-        dst.crs = {'init': 'epsg:32617'}
-        dst.nodata = 1.0
-
-    # Double check.
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32617'}
-        assert src.nodata == 1.0
-
-    # The test.
-    templatefile = 'tests/data/RGB.byte.tif'
-    result = runner.invoke(
-        main_group,
-        ['edit-info', inputfile, '--like', templatefile, '--nodata', 'like'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32617'}
-        assert src.nodata == 0.0
-
-
-def test_edit_all_like(data):
-    runner = CliRunner()
-
-    inputfile = str(data.join('RGB.byte.tif'))
-    with rasterio.open(inputfile, 'r+') as dst:
-        dst.crs = {'init': 'epsg:32617'}
-        dst.nodata = 1.0
-
-    # Double check.
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32617'}
-        assert src.nodata == 1.0
-
-    templatefile = 'tests/data/RGB.byte.tif'
-    result = runner.invoke(
-        main_group, ['edit-info', inputfile, '--like', templatefile, '--all'])
-    assert result.exit_code == 0
-    with rasterio.open(inputfile) as src:
-        assert src.crs == {'init': 'epsg:32618'}
-        assert src.nodata == 0.0
+with rasterio.Env() as env:
+    HAVE_NETCDF = 'NetCDF' in env.drivers().keys()
 
 
 def test_env():
@@ -275,10 +26,14 @@ def test_env():
 
 
 def test_info_err():
+    """Trying to get info of a directory raises an exception"""
     runner = CliRunner()
     result = runner.invoke(
         main_group, ['info', 'tests'])
-    assert result.exit_code == 1
+    assert result.exit_code != 0
+    assert result.exception
+    # Note: text of exception changed after 2.1, don't test on full string
+    assert 'not' in result.output and ' a valid input file' in result.output
 
 
 def test_info():
@@ -286,7 +41,54 @@ def test_info():
     result = runner.invoke(
         main_group, ['info', 'tests/data/RGB.byte.tif'])
     assert result.exit_code == 0
-    assert '"count": 3' in result.output
+    info = json.loads(result.output)
+    assert info['count'] == 3
+    assert info['dtype'] == 'uint8'
+    assert info['crs'] == 'EPSG:32618'
+
+    result = runner.invoke(
+        main_group, ['info', 'tests/data/float.tif'])
+    assert result.exit_code == 0
+    info = json.loads(result.output)
+    assert info['count'] == 1
+    assert info['dtype'] == 'float64'
+    assert info['crs'] is None
+
+
+def test_info_units():
+    """Find a units item"""
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group, ['info', 'tests/data/RGB.byte.tif'])
+    assert result.exit_code == 0
+    assert '"units": [null, null, null]' in result.output
+
+
+def test_info_indexes():
+    """Find an indexes item"""
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group, ['info', 'tests/data/RGB.byte.tif'])
+    assert result.exit_code == 0
+    assert '"indexes": [1, 2, 3]' in result.output
+
+
+def test_info_descriptions():
+    """Find description items"""
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group, ['info', 'tests/data/RGB.byte.tif'])
+    assert result.exit_code == 0
+    assert '"descriptions"' in result.output
+
+
+def test_info_mask_flags():
+    """Find description items"""
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group, ['info', 'tests/data/RGB.byte.tif'])
+    assert result.exit_code == 0
+    assert '"mask_flags": [["nodata"], ["nodata"], ["nodata"]]' in result.output
 
 
 def test_info_verbose():
@@ -596,7 +398,8 @@ def test_bounds_seq_rs():
     ])
     assert result.exit_code == 0
     assert result.output == (
-        '\x1e[-78.96, 23.56, -76.57, 25.55]\n\x1e[-78.96, 23.56, -76.57, 25.55]\n')
+        '\x1e[-78.96, 23.56, -76.57, 25.55]\n'
+        '\x1e[-78.96, 23.56, -76.57, 25.55]\n')
 
 
 def test_insp():
@@ -608,7 +411,7 @@ def test_insp():
 def test_insp_err():
     runner = CliRunner()
     result = runner.invoke(main_group, ['insp', 'tests'])
-    assert result.exit_code == 1
+    assert result.exit_code != 0
 
 
 def test_info_checksums():
@@ -628,8 +431,14 @@ def test_info_checksums_only():
     assert result.output.strip() == '29131'
 
 
-def test_bad_interpreter():
-    from rasterio.rio.insp import main
-    with rasterio.open("tests/data/RGB.byte.tif", 'r') as src:
-        with pytest.raises(ValueError):
-            main("Test banner", src, "PHP")
+@requires_gdal21(reason="NetCDF requires GDAL 2.1+")
+@pytest.mark.skipif(not HAVE_NETCDF,
+                    reason="GDAL not compiled with NetCDF driver.")
+def test_info_subdatasets():
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group,
+        ['info', 'netcdf:tests/data/RGB.nc', '--subdatasets'])
+    assert result.exit_code == 0
+    assert len(result.output) == 93
+    assert result.output.startswith('netcdf:tests/data/RGB.nc:Band1')
