@@ -1,25 +1,30 @@
+"""Unittests for $ rio merge"""
+
+
 import sys
 import os
 import logging
-import click
-import numpy as np
+
+import affine
 from click.testing import CliRunner
+import numpy as np
 from pytest import fixture
+import pytest
 
 import rasterio
-from rasterio.rio.merge import merge
+from rasterio.merge import merge
+from rasterio.rio.main import main_group
 from rasterio.transform import Affine
 
-
-logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
-
+from .conftest import requires_gdal22
 
 # Fixture to create test datasets within temporary directory
 @fixture(scope='function')
 def test_data_dir_1(tmpdir):
     kwargs = {
         "crs": {'init': 'epsg:4326'},
-        "transform": (-114, 0.2, 0, 46, 0, -0.2),
+        "transform": affine.Affine(0.2, 0, -114,
+                                   0, -0.2, 46),
         "count": 1,
         "dtype": rasterio.uint8,
         "driver": "GTiff",
@@ -45,7 +50,8 @@ def test_data_dir_1(tmpdir):
 def test_data_dir_2(tmpdir):
     kwargs = {
         "crs": {'init': 'epsg:4326'},
-        "transform": (-114, 0.2, 0, 46, 0, -0.1),
+        "transform": affine.Affine(0.2, 0, -114,
+                                   0, -0.2, 46),
         "count": 1,
         "dtype": rasterio.uint8,
         "driver": "GTiff",
@@ -67,6 +73,33 @@ def test_data_dir_2(tmpdir):
     return tmpdir
 
 
+@fixture(scope='function')
+def test_data_dir_3(tmpdir):
+    kwargs = {
+        "crs": {'init': 'epsg:4326'},
+        "transform": affine.Affine(0.2, 0, -114,
+                                   0, -0.2, 46),
+        "count": 2,  # important: band count > 1
+        "dtype": rasterio.uint8,
+        "driver": "GTiff",
+        "width": 10,
+        "height": 10,
+        "nodata": 1
+    }
+
+    with rasterio.open(str(tmpdir.join('b.tif')), 'w', **kwargs) as dst:
+        data = np.ones((2, 10, 10), dtype=rasterio.uint8)
+        data[:, 0:6, 0:6] = 255
+        dst.write(data)
+
+    with rasterio.open(str(tmpdir.join('a.tif')), 'w', **kwargs) as dst:
+        data = np.ones((2, 10, 10), dtype=rasterio.uint8)
+        data[:, 4:8, 4:8] = 254
+        dst.write(data)
+
+    return tmpdir
+
+
 def test_merge_with_colormap(test_data_dir_1):
     outputname = str(test_data_dir_1.join('merged.tif'))
     inputs = [str(x) for x in test_data_dir_1.listdir()]
@@ -77,7 +110,7 @@ def test_merge_with_colormap(test_data_dir_1):
         src.write_colormap(1, {0: (255, 0, 0, 255), 255: (0, 0, 0, 0)})
 
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -87,12 +120,14 @@ def test_merge_with_colormap(test_data_dir_1):
         assert cmap[255] == (0, 0, 0, 255)
 
 
+@requires_gdal22(
+    reason="This test is sensitive to pixel values and requires GDAL 2.2+")
 def test_merge_with_nodata(test_data_dir_1):
     outputname = str(test_data_dir_1.join('merged.tif'))
     inputs = [str(x) for x in test_data_dir_1.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
     with rasterio.open(outputname) as out:
@@ -104,23 +139,40 @@ def test_merge_with_nodata(test_data_dir_1):
         assert np.all(data == expected)
 
 
-def test_merge_warn(test_data_dir_1):
+def test_merge_error(test_data_dir_1):
+    """A nodata value outside the valid range results in an error"""
     outputname = str(test_data_dir_1.join('merged.tif'))
     inputs = [str(x) for x in test_data_dir_1.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname] + ['--nodata', '-1'])
+    result = runner.invoke(
+        main_group, ['merge'] + inputs + [outputname] + ['--nodata', '-1'])
+    assert result.exit_code == -1
+
+
+def test_merge_bidx(test_data_dir_3):
+    outputname = str(test_data_dir_3.join('merged.tif'))
+    inputs = [str(x) for x in test_data_dir_3.listdir()]
+    inputs.sort()
+    runner = CliRunner()
+    result = runner.invoke(
+        main_group, ['merge'] + inputs + [outputname] + ['--bidx', '1'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
-    assert "using the --nodata option for better results" in result.output
+    with rasterio.open(inputs[0]) as src:
+        assert src.count > 1
+    with rasterio.open(outputname) as out:
+        assert out.count == 1
 
 
+@requires_gdal22(
+    reason="This test is sensitive to pixel values and requires GDAL 2.2+")
 def test_merge_without_nodata(test_data_dir_2):
     outputname = str(test_data_dir_2.join('merged.tif'))
     inputs = [str(x) for x in test_data_dir_2.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
     with rasterio.open(outputname) as out:
@@ -136,33 +188,31 @@ def test_merge_output_exists(tmpdir):
     outputname = str(tmpdir.join('merged.tif'))
     runner = CliRunner()
     result = runner.invoke(
-        merge,
-        ['tests/data/RGB.byte.tif', outputname])
+        main_group, ['merge', 'tests/data/RGB.byte.tif', outputname])
     assert result.exit_code == 0
     result = runner.invoke(
-        merge,
-        ['tests/data/RGB.byte.tif', outputname])
+        main_group, ['merge', 'tests/data/RGB.byte.tif', outputname])
     assert os.path.exists(outputname)
     with rasterio.open(outputname) as out:
         assert out.count == 3
 
 
 def test_merge_output_exists_without_nodata_fails(test_data_dir_2):
-    """Fails without --force-overwrite"""
+    """Fails without --overwrite"""
     runner = CliRunner()
     result = runner.invoke(
-        merge,
-        [str(test_data_dir_2.join('a.tif')),
+        main_group, [
+            'merge', str(test_data_dir_2.join('a.tif')),
             str(test_data_dir_2.join('b.tif'))])
     assert result.exit_code == 1
 
 
 def test_merge_output_exists_without_nodata(test_data_dir_2):
-    """Succeeds with --force-overwrite"""
+    """Succeeds with --overwrite"""
     runner = CliRunner()
     result = runner.invoke(
-        merge,
-        ['--force-overwrite', str(test_data_dir_2.join('a.tif')),
+        main_group, [
+            'merge', '--overwrite', str(test_data_dir_2.join('a.tif')),
             str(test_data_dir_2.join('b.tif'))])
     assert result.exit_code == 0
 
@@ -170,8 +220,7 @@ def test_merge_output_exists_without_nodata(test_data_dir_2):
 def test_merge_err():
     runner = CliRunner()
     result = runner.invoke(
-        merge,
-        ['tests'])
+        main_group, ['merge', 'tests'])
     assert result.exit_code == 1
 
 
@@ -179,8 +228,9 @@ def test_format_jpeg(tmpdir):
     outputname = str(tmpdir.join('stacked.jpg'))
     runner = CliRunner()
     result = runner.invoke(
-        merge,
-        ['tests/data/RGB.byte.tif', outputname, '--format', 'JPEG'])
+        main_group, [
+            'merge', 'tests/data/RGB.byte.tif', outputname,
+            '--format', 'JPEG'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
 
@@ -191,7 +241,8 @@ def test_format_jpeg(tmpdir):
 def test_data_dir_overlapping(tmpdir):
     kwargs = {
         "crs": {'init': 'epsg:4326'},
-        "transform": (-114, 0.2, 0, 46, 0, -0.2),
+        "transform": affine.Affine(0.2, 0, -114,
+                                   0, -0.2, 46),
         "count": 1,
         "dtype": rasterio.uint8,
         "driver": "GTiff",
@@ -204,7 +255,8 @@ def test_data_dir_overlapping(tmpdir):
         data = np.ones((10, 10), dtype=rasterio.uint8)
         dst.write(data, indexes=1)
 
-    kwargs['transform'] = (-113, 0.2, 0, 45, 0, -0.2)
+    kwargs['transform'] = affine.Affine(0.2, 0, -113,
+                                        0, -0.2, 45)
     with rasterio.open(str(tmpdir.join('nw.tif')), 'w', **kwargs) as dst:
         data = np.ones((10, 10), dtype=rasterio.uint8) * 2
         dst.write(data, indexes=1)
@@ -217,7 +269,7 @@ def test_merge_overlapping(test_data_dir_overlapping):
     inputs = [str(x) for x in test_data_dir_overlapping.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
     with rasterio.open(outputname) as out:
@@ -236,7 +288,8 @@ def test_merge_overlapping(test_data_dir_overlapping):
 def test_data_dir_float(tmpdir):
     kwargs = {
         "crs": {'init': 'epsg:4326'},
-        "transform": (-114, 0.2, 0, 46, 0, -0.2),
+        "transform": affine.Affine(0.2, 0, -114,
+                                   0, -0.2, 46),
         "count": 1,
         "dtype": rasterio.float64,
         "driver": "GTiff",
@@ -257,12 +310,15 @@ def test_data_dir_float(tmpdir):
     return tmpdir
 
 
+@requires_gdal22(
+    reason="This test is sensitive to pixel values and requires GDAL 2.2+")
 def test_merge_float(test_data_dir_float):
     outputname = str(test_data_dir_float.join('merged.tif'))
     inputs = [str(x) for x in test_data_dir_float.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname] + ['--nodata', '-1.5'])
+    result = runner.invoke(
+        main_group, ['merge'] + inputs + [outputname] + ['--nodata', '-1.5'])
     assert result.exit_code == 0
     assert os.path.exists(outputname)
     with rasterio.open(outputname) as out:
@@ -312,12 +368,12 @@ def tiffs(tmpdir):
     return tmpdir
 
 
-def test_merge_tiny(tiffs):
+def test_merge_tiny_base(tiffs):
     outputname = str(tiffs.join('merged.tif'))
     inputs = [str(x) for x in tiffs.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
 
     # Output should be
@@ -329,6 +385,7 @@ def test_merge_tiny(tiffs):
 
     with rasterio.open(outputname) as src:
         data = src.read()
+        print(data)
         assert (data[0][0:2, 1] == 120).all()
         assert (data[0][0:2, 2:4] == 90).all()
         assert data[0][2][1] == 60
@@ -340,7 +397,7 @@ def test_merge_tiny_output_opt(tiffs):
     inputs = [str(x) for x in tiffs.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + ['-o', outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + ['-o', outputname])
     assert result.exit_code == 0
 
     # Output should be
@@ -358,45 +415,29 @@ def test_merge_tiny_output_opt(tiffs):
         assert data[0][3][0] == 40
 
 
+@requires_gdal22(
+    reason="This test is sensitive to pixel values and requires GDAL 2.2+")
+@pytest.mark.xfail(sys.version_info < (3,),
+                   reason="Test is sensitive to rounding behavior")
 def test_merge_tiny_res_bounds(tiffs):
     outputname = str(tiffs.join('merged.tif'))
     inputs = [str(x) for x in tiffs.listdir()]
     inputs.sort()
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname, '--res', 2, '--bounds', 1, 0, 5, 4])
+    result = runner.invoke(
+        main_group, ['merge'] + inputs + [outputname, '--res', 2, '--bounds', '1, 0, 5, 4'])
     assert result.exit_code == 0
 
     # Output should be
-    # [[[120  90]
-    #   [ 40   0]]]
+    # [[[0  90]
+    #   [0   0]]]
 
     with rasterio.open(outputname) as src:
         data = src.read()
         print(data)
-        assert data[0, 0, 0] == 120
+        assert data[0, 0, 0] == 0
         assert data[0, 0, 1] == 90
-        assert data[0, 1, 0] == 40
-        assert data[0, 1, 1] == 0
-
-
-def test_merge_tiny_res_high_precision(tiffs):
-    outputname = str(tiffs.join('merged.tif'))
-    inputs = [str(x) for x in tiffs.listdir()]
-    inputs.sort()
-    runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname, '--res', 2, '--precision', 15])
-    assert result.exit_code == 0
-
-    # Output should be
-    # [[[120  90]
-    #   [ 40   0]]]
-
-    with rasterio.open(outputname) as src:
-        data = src.read()
-        print(data)
-        assert data[0, 0, 0] == 120
-        assert data[0, 0, 1] == 90
-        assert data[0, 1, 0] == 40
+        assert data[0, 1, 0] == 0
         assert data[0, 1, 1] == 0
 
 
@@ -409,7 +450,7 @@ def test_merge_rgb(tmpdir):
         'tests/data/rgb3.tif',
         'tests/data/rgb4.tif']
     runner = CliRunner()
-    result = runner.invoke(merge, inputs + [outputname])
+    result = runner.invoke(main_group, ['merge'] + inputs + [outputname])
     assert result.exit_code == 0
 
     with rasterio.open(outputname) as src:
@@ -417,8 +458,7 @@ def test_merge_rgb(tmpdir):
 
 
 def test_merge_tiny_intres(tiffs):
-    from rasterio.merge import merge
     inputs = [str(x) for x in tiffs.listdir()]
     inputs.sort()
-    sources = [rasterio.open(x) for x in inputs]
-    merge(sources, res=2)
+    datasets = [rasterio.open(x) for x in inputs]
+    merge(datasets, res=2)
